<?php

class nc_payment_system_zpayment extends nc_payment_system {

    const ERROR_MERCHANT_ID = NETCAT_MODULE_PAYMENT_ZPAYMENT_ERROR_MERCHANT_ID_IS_NOT_VALID;
    const ERROR_SECRET_KEY = NETCAT_MODULE_PAYMENT_ZPAYMENT_ERROR_SECRET_KEY_IS_NOT_VALID;
    const ERROR_CURRENCY = NETCAT_MODULE_PAYMENT_ZPAYMENT_ERROR_CURRENCY_IS_NOT_VALID;
    const ERROR_SIGN_IS_NOT_VALID = NETCAT_MODULE_PAYMENT_ZPAYMENT_ERROR_SIGN_IS_NOT_VALID;

    const TARGET_URL = "https://z-payment.com/merchant.php";

    protected $automatic = TRUE;

    // принимаемые валюты
    public $accepted_currencies = array('USD', 'EUR', 'RUR', 'RUR');

    // параметры сайта в платежной системе
    protected $settings = array(
        'shop_id'	=> null,
        'merchant_key'	=> null,
		'initialization_password'	=> null,
    );

    // передаваемые параметры
    protected $request_parameters = array( // 'InvId' => null,
        // 'InvDesc' => null,
    );

    // получаемые параметры
    protected $callback_response = array(
        'InvId' => null,
        'OutSum' => null,
    );

    /**
     *
     */
    public function execute_payment_request(nc_payment_invoice $invoice) {
		
		$netshop = nc_netshop::get_instance();
		$order = $netshop->load_order($invoice->get('order_id'));
		$strDescription = '';
		foreach($order->get_items() as $item){
			$strDescription .= $item->get('Vendor')." ";
			$strDescription .= $item->get('Name').";";
			if($item->get('Qty') > 1)
				$strDescription .= " * ".$item->get('Qty');
		}
		$zpsign = md5($this->get_setting('shop_id').$invoice->get('order_id').$invoice->get_amount("%0.2F").$this->get_setting('initialization_password'));
		$arrFields = array(
            'LMI_PAYEE_PURSE'	=> $this->get_setting('shop_id'),
            'LMI_PAYMENT_NO'		=> $invoice->get('order_id'),
            'LMI_PAYMENT_AMOUNT'			=> $invoice->get_amount("%0.2F"),
            'LMI_PAYMENT_DESC'	=> mb_substr($strDescription, 0, 255, "UTF-8"),
			'ZP_SIGN'	=> $zpsign,

        );

		
		if(preg_match('/^.+@.+\..+$/', @$invoice->get('customer_email'))){
			$arrFields['CLIENT_MAIL'] = $invoice->get('customer_email');
		}
	
        ob_end_clean();
        $strForm = "
            <html>
              <body>
                    <form action='" . nc_payment_system_zpayment::TARGET_URL . "' method='POST'>".
                     $this->make_inputs($arrFields)
					."
					<input type='submit' value='Оплатить'>
                </form>
                <script>
                  document.forms[0].submit();
                </script>
              </body>
            </html>";
        echo $strForm;
        exit;
    }

    /**
     * @param nc_payment_invoice $invoice
     */
    public function on_response(nc_payment_invoice $invoice = null) {
    }

    /**
     *
     */
    public function validate_payment_request_parameters() {
		if (!$this->get_setting('shop_id')) {
            $this->add_error(nc_payment_system_zpayment::ERROR_MERCHANT_ID);
        }
		elseif(!$this->get_setting('merchant_key')) {
            $this->add_error(nc_payment_system_zpayment::ERROR_SECRET_KEY);
        }
		
    }

    /**
     * @param nc_payment_invoice $invoice
     */
    public function validate_payment_callback_response(nc_payment_invoice $invoice = null) {	
	
	    if($this->get_response_value('LMI_PREREQUEST')==1) {
		    echo 'YES';
		    die();
        }

	    $LMI_PAYMENT_NO = $this->get_response_value('LMI_PAYMENT_NO');
		$LMI_PAYEE_PURSE = $this->get_response_value('LMI_PAYEE_PURSE');
		$LMI_PAYMENT_AMOUNT = $this->get_response_value('LMI_PAYMENT_AMOUNT');
		$LMI_SECRET_KEY = $this->get_response_value('LMI_SECRET_KEY');
		$LMI_MODE = $this->get_response_value('LMI_MODE');
		$LMI_SYS_INVS_NO = $this->get_response_value('LMI_SYS_INVS_NO');
		$LMI_SYS_TRANS_NO = $this->get_response_value('LMI_SYS_TRANS_NO');
		$LMI_SYS_TRANS_DATE = $this->get_response_value('LMI_SYS_TRANS_DATE');
		$LMI_PAYER_PURSE = $this->get_response_value('LMI_PAYER_PURSE');
		$LMI_PAYER_WM = $this->get_response_value('LMI_PAYER_WM');
		$LMI_HASH = $this->get_response_value('LMI_HASH');
		
		$shop_id = $this->get_setting('shop_id');
		$shopSecret = $this->get_setting('merchant_key');
		$amount = $invoice->get('amount');
		
		$err=FALSE;
		
		if ($shop_id!=$LMI_PAYEE_PURSE) {
		    $err=TRUE;
			$errText="Shop Id mismatch";
		}
		
		if ($amount != 	$LMI_PAYMENT_AMOUNT) {
		    $err=TRUE;
			$errText="Amount mismatch";
		}
		
		if(isset($LMI_SECRET_KEY)) {
            if($LMI_SECRET_KEY!=$shopSecret) {
                $err = TRUE;
				$errText='secret key mismatch';
		    }
	    } else {
        $CalcHash = md5($LMI_PAYEE_PURSE.$LMI_PAYMENT_AMOUNT.$LMI_PAYMENT_NO.$LMI_MODE.$LMI_SYS_INVS_NO.$LMI_SYS_TRANS_NO.$LMI_SYS_TRANS_DATE.$shopSecret.$LMI_PAYER_PURSE.$LMI_PAYER_WM);
		    if($LMI_HASH != strtoupper($CalcHash)) {
                $err = TRUE;
				$errText='hash mismatch';
	        }
        }
	
		if (!$err) {
		    $strResponseStatus = 'ok';
			$strResponseDescription = "Оплата принята";
			if ($invoice) {		
     			$invoice->set('status', nc_payment_invoice::STATUS_SUCCESS);
				$invoice->save();
				$this->on_payment_success($invoice);
				echo 'YES';
			}
		}
		
		



    }

    public function load_invoice_on_callback() {
		return $this->load_invoice($this->get_response_value('LMI_PAYMENT_NO'));
    }
}

